package com.david.oviedotourist.activities;

import android.content.Intent;
import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.text.Html;
import android.text.method.LinkMovementMethod;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.david.oviedotourist.R;
import com.david.oviedotourist.pojo.GeoCoordinate;
import com.david.oviedotourist.pojo.Site;
import com.david.oviedotourist.utilities.MyNotifications;
import com.david.oviedotourist.utilities.MyScrollView;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;


/**
 * Created by David on 08/11/2016.
 */

public class EventDetailActivity extends AppCompatActivity {


    //Data Management//
    private String id, name, date, time, image, type;
    private GeoCoordinate coordinates;
    private ImageView eventImage, eventIcon;
    private TextView eventName, eventAddress, eventDescription, eventDate, eventTime, eventInfo;


    //FloatingActionButtons//
    private FloatingActionButton fabToGoogleMaps;

    private final static String EVENT_DETAILS_CHILD = "eventsDetails/";


    //////////////////////////////Activity Lifecycle///////////////////////////////

    @Override
    protected void onCreate(Bundle saveInstanceState) {
        super.onCreate(saveInstanceState);
        setContentView(R.layout.event_detail);

        Bundle bundle = getIntent().getExtras();
        id = bundle.getString("id", "");
        name = bundle.getString("name", "");
        date = bundle.getString("date", "");
        time = bundle.getString("time", "");
        image = bundle.getString("image", "");
        type = bundle.getString("type", "");


        /////CUSTOM BARS AND COLORS/////

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);

        toolbar.setBackgroundColor(checkTypeColor(type, false));
        setSupportActionBar(toolbar);

        if (getSupportActionBar() != null) {
            ActionBar actionBar = getSupportActionBar();
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setDisplayShowHomeEnabled(true);
            actionBar.setTitle(name);
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Window window = getWindow();
            window.setStatusBarColor(checkTypeColor(type, true));
        }

        /////  /////

        final LinearLayout loadingLayout = (LinearLayout) findViewById(R.id.ly_loading);

        eventImage = (ImageView) findViewById(R.id.event_image);
        eventIcon = (ImageView) findViewById(R.id.event_icon);
        eventName = (TextView) findViewById(R.id.event_name);
        eventAddress = (TextView) findViewById(R.id.event_address);
        eventDescription = (TextView) findViewById(R.id.event_description);
        eventDate = (TextView) findViewById(R.id.event_date);
        eventTime = (TextView) findViewById(R.id.event_time);
        eventInfo = (TextView) findViewById(R.id.event_info);
        fabToGoogleMaps = (FloatingActionButton) findViewById(R.id.fab_google_maps);

        DatabaseReference databaseReference = FirebaseDatabase.getInstance().
                getReference().child(EVENT_DETAILS_CHILD + id);
        databaseReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.exists()) {
                    Site site = dataSnapshot.getValue(Site.class);

                    loadingLayout.setVisibility(View.INVISIBLE);

                    eventIcon.setImageDrawable(ContextCompat.getDrawable(
                            EventDetailActivity.this, checkTypeIcon(type)));
                    eventName.setText(name);
                    eventAddress.setText(site.getAddress());
                    eventDescription.setText(Html.fromHtml(site.getDescription()));
                    eventDate.setText(date);
                    eventTime.setText(time);
                    eventInfo.setText(Html.fromHtml(site.getEvent().getInfo()));
                    eventInfo.setMovementMethod(LinkMovementMethod.getInstance());
                    coordinates = site.getsCoordinate();

                    Glide.with(EventDetailActivity.this)
                            .load(image)
                            .diskCacheStrategy(DiskCacheStrategy.SOURCE)
                            .fitCenter()
                            .centerCrop()
                            .into(eventImage);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                MyNotifications.createToastNotification(EventDetailActivity.this,
                        R.string.server_conexion_cancelled, MyNotifications.SHORT_DURATION);
            }
        });

        Animation toGoogleMaps = AnimationUtils.loadAnimation(this, R.anim.zoom_in);
        toGoogleMaps.setStartOffset(200);
        fabToGoogleMaps.startAnimation(toGoogleMaps);
        fabToGoogleMaps.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                fabToGoogleMaps.startAnimation(AnimationUtils.loadAnimation(
                        EventDetailActivity.this, R.anim.zoom_out));
                Intent intent = new Intent(EventDetailActivity.this, VirtualTourActivity.class);
                intent.putExtra("isEvent", true);
                intent.putExtra("id", id);
                intent.putExtra("name", name);
                intent.putExtra("eventLatitude", coordinates.getLatitude());
                intent.putExtra("eventLongitude", coordinates.getLongitude());
                intent.putExtra("type", type);
                startActivity(intent);
            }
        });

        final MyScrollView scroll = (MyScrollView) findViewById(R.id.scroll_view);
        scroll.setOnScrollViewListener(new MyScrollView.OnScrollViewListener() {
            @Override
            public void onScrollChanged(int x, int y, int previousX, int previousY) {
                int lastPositionY, previousPositionY;
                if (y <= 0 || previousY <= 0) {
                    lastPositionY = 0;
                    previousPositionY = 0;
                } else {
                    lastPositionY = y;
                    previousPositionY = previousY;
                }
                if (lastPositionY > previousPositionY) {
                    fabToGoogleMaps.hide();
                } else {
                    fabToGoogleMaps.show();
                }
            }
        });

    }

    /////////////////////////////////////////////////////////////////////////////////////

    //////////////////////////////////ActionBar Options//////////////////////////////////

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.secondary_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                return true;
            case R.id.menu_share:
                shareEvent(name);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private int checkTypeColor(String string, boolean statusBar) {
        switch (string) {
            case "CULTURE":
                return statusBar ? ContextCompat.getColor(this, R.color.cultureDark)
                        : ContextCompat.getColor(this, R.color.culture);
            case "NATURE":
                return statusBar ? ContextCompat.getColor(this, R.color.natureDark)
                        : ContextCompat.getColor(this, R.color.nature);
            case "ENTERTAINMENT":
                return statusBar ? ContextCompat.getColor(this, R.color.entertainmentDark)
                        : ContextCompat.getColor(this, R.color.entertainment);
            case "GASTRONOMY":
                return statusBar ? ContextCompat.getColor(this, R.color.gastronomyDark)
                        : ContextCompat.getColor(this, R.color.gastronomy);
        }
        return ContextCompat.getColor(this, R.color.colorPrimaryDark);
    }

    private void shareEvent(String name) {
        Intent intent = new Intent(Intent.ACTION_SEND);
        intent.setType("text/plain");
        intent.putExtra(Intent.EXTRA_TEXT, getString(R.string.event_text) +
                " " + name);
        startActivity(Intent.createChooser(intent, getString(R.string.send_to)));
    }

    ////////////////////////////////////////////////////////////////////////////////////////

    private int checkTypeIcon(String string) {
        switch (string) {
            case "CULTURE":
                return R.drawable.ic_culture;
            case "NATURE":
                return R.drawable.ic_nature;
            case "ENTERTAINMENT":
                return R.drawable.ic_entertainment;
            case "GASTRONOMY":
                return R.drawable.ic_gastronomy;
        }
        return -1;
    }
}
