package com.david.oviedotourist.activities;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.AsyncTask;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.provider.Settings;
import android.support.v4.content.LocalBroadcastManager;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.StaggeredGridLayoutManager;
import android.view.MenuItem;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.david.oviedotourist.R;
import com.david.oviedotourist.adapters.SiteAdapter;
import com.david.oviedotourist.pojo.GeoCoordinate;
import com.david.oviedotourist.pojo.Site;
import com.david.oviedotourist.services.LocationService;
import com.david.oviedotourist.utilities.MyNotifications;
import com.david.oviedotourist.utilities.MyPermissions;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;


import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Created by David on 12/11/2016.
 */

public class LiveTourActivity extends AppCompatActivity {


    ///// RECYCLER VIEW /////
    private RecyclerView recyclerView;
    private RecyclerView.LayoutManager layoutManager;
    private SiteAdapter siteAdapter;

    ///// FIREBASE DATABASE /////
    private static final String SITES_CHILD = "sites/";
    private DatabaseReference databaseReference;
    private boolean isDatabaseLoaded;

    private ProgressBar progressBar;
    private TextView loadingText;

    private static List<Site> siteList;
    private static List<Site> closestSites;
    private static GeoCoordinate myPosition;
    private BroadcastReceiver receiver;

    private SharedPreferences preferences;
    private double maximunDistance;

    //////////////////////////////Activity Lifecycle///////////////////////////////
    @Override
    protected void onCreate(Bundle saveInstanceState) {
        super.onCreate(saveInstanceState);
        setContentView(R.layout.live_tour);

        ///// ACTION BAR /////

        if (getSupportActionBar() != null){
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
            getSupportActionBar().setDisplayShowHomeEnabled(true);
        }

        /////    /////

        preferences = PreferenceManager.getDefaultSharedPreferences(this);
        maximunDistance = (double) preferences.getInt("prf_seekbar", 50);

        isDatabaseLoaded = false;

        progressBar = (ProgressBar) findViewById(R.id.progressBar);
        loadingText = (TextView) findViewById(R.id.tv_loading);

        siteList = new ArrayList<>();
        closestSites = new ArrayList<>();

        recyclerView = (RecyclerView) findViewById(R.id.image_content);
        siteAdapter = new SiteAdapter(LiveTourActivity.this, closestSites,
                R.layout.image_element, false, true, false);
        StaggeredGridLayoutManager staggeredGridLayoutManager = new StaggeredGridLayoutManager(2,
                StaggeredGridLayoutManager.VERTICAL);
        staggeredGridLayoutManager.setGapStrategy(
                StaggeredGridLayoutManager.GAP_HANDLING_MOVE_ITEMS_BETWEEN_SPANS);
        layoutManager = staggeredGridLayoutManager;
        recyclerView.setAdapter(siteAdapter);
        recyclerView.setLayoutManager(layoutManager);

        startService(new Intent(this, LocationService.class));

        receiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                Bundle bundle = intent.getExtras();
                switch (intent.getAction()){
                    case LocationService.PROVIDER_AVAILABLE:
                        boolean provider = bundle.getBoolean("provider");

                        if (!provider) {
                            MyPermissions.requestServiceActivation(LiveTourActivity.this,
                                    R.string.error_title, R.string.provider_disable,
                                    Settings.ACTION_LOCATION_SOURCE_SETTINGS);
                        }
                        break;
                    case LocationService.NEW_LOCATION_AVAILABLE:
                        double latitude = bundle.getDouble("myLatitude");
                        double longitude = bundle.getDouble("myLongitude");
                        GeoCoordinate newPosition = new GeoCoordinate(latitude, longitude);

                        if (newPosition != myPosition && isDatabaseLoaded) {
                            myPosition = newPosition;
                            UpdateClosestSites updateClosestSites = new UpdateClosestSites();
                            updateClosestSites.execute();
                        }
                        break;
                }
            }
        };

        databaseReference = FirebaseDatabase.getInstance().getReference()
                .child(SITES_CHILD);
        databaseReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.exists()) {
                    Iterable<DataSnapshot> references = dataSnapshot.getChildren();
                    while (references.iterator().hasNext()) {
                        DataSnapshot nextDataReference = references.iterator().next();
                        Site site = nextDataReference.getValue(Site.class);
                        siteList.add(site);
                    }
                    isDatabaseLoaded = !siteList.isEmpty();
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                MyNotifications.createToastNotification(LiveTourActivity.this,
                        R.string.server_conexion_cancelled,
                        MyNotifications.LONG_DURATION);
            }
        });
    }

    @Override
    protected void onStart() {
        super.onStart();
        LocalBroadcastManager.getInstance(this).registerReceiver(receiver,
                new IntentFilter(LocationService.NEW_LOCATION_AVAILABLE));
        LocalBroadcastManager.getInstance(this).registerReceiver(receiver,
                new IntentFilter(LocationService.PROVIDER_AVAILABLE));
    }

    @Override
    protected void onRestart() {
        super.onRestart();
        startService(new Intent(this, LocationService.class));
    }

    @Override
    protected void onStop() {
        LocalBroadcastManager.getInstance(this).unregisterReceiver(receiver);
        stopService(new Intent(this, LocationService.class));
        super.onStop();
    }

    /////////////////////////////////////////////////////////////////////////////////

    /////////////////////////////////Site Management/////////////////////////////////


    private void checkClosestSites(GeoCoordinate myPosition, List<Site> siteList) {

        Set<String> userSelectedTypes = preferences.getStringSet("prf_site_type", null);

        closestSites.clear();

        if (myPosition!= null) {
            for (Site site : siteList) {
                if (site.getsCoordinate().distance(myPosition) <= maximunDistance) {
                    if (userSelectedTypes != null
                            && userSelectedTypes.contains(site.getType().toString())) {
                        closestSites.add(site);
                    }
                }
            }
        }
    }

    private void loadClosestSites() {

        siteAdapter.swap(closestSites);

        siteAdapter.setItemOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int position = recyclerView.getChildAdapterPosition(view);
                Site site = closestSites.get(position);

                Intent intent = new Intent(LiveTourActivity.this,
                        SiteDetailActivity.class);
                intent.putExtra("id", site.getId());
                intent.putExtra("name", site.getName());
                intent.putExtra("siteLatitude", site.getsCoordinate().getLatitude());
                intent.putExtra("siteLongitude", site.getsCoordinate().getLongitude());
                intent.putExtra("distance", myPosition.distance(site.getsCoordinate()));
                intent.putExtra("type", site.getType().toString());
                startActivity(intent);
            }
        });
    }

    class UpdateClosestSites extends AsyncTask<Void, Void, Void> {

        @Override
        protected void onPreExecute() {
            progressBar.setVisibility(ProgressBar.VISIBLE);
            loadingText.setVisibility(TextView.VISIBLE);
        }

        @Override
        protected Void doInBackground(Void... voids) {
            checkClosestSites(myPosition, siteList);
            return null;
        }

        @Override
        protected void onPostExecute(Void voids) {

            if (myPosition != null) {
                progressBar.setVisibility(ProgressBar.INVISIBLE);
                loadingText.setVisibility(TextView.INVISIBLE);
            }

            loadClosestSites();

            if (closestSites.size() == 0 && isDatabaseLoaded) {
                MyNotifications.createToastNotification(LiveTourActivity.this,
                        R.string.no_closest_place, MyNotifications.SHORT_DURATION);
            }
        }
    }
    /////////////////////////////////////////////////////////////////////////////////

    //////////////////////////////////ActionBar Options//////////////////////////////////

    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch(item.getItemId()){
            case android.R.id.home:
                finish();
        }
        return super.onOptionsItemSelected(item);
    }
}
