package com.david.oviedotourist.activities;

import android.content.Intent;
import android.os.Bundle;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.ProgressBar;
import android.widget.TextView;

import com.david.oviedotourist.R;
import com.david.oviedotourist.adapters.SiteAdapter;
import com.david.oviedotourist.data.MySitesDatabase;
import com.david.oviedotourist.pojo.Site;
import com.david.oviedotourist.utilities.MyNotifications;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by David on 29/11/2016.
 */

public class MySitesActivity extends AppCompatActivity {


    ///// LOCAL DATABASE /////
    private MySitesDatabase myDatabase;
    private String myList;
    private List<String> mySitesId;

    ///// FIREBASE DATABASE /////
    private static final String SITES_CHILD = "sites/";
    private DatabaseReference databaseReference;
    private List<Site> mySitesList;

    ///// RECYCLER VIEW /////
    private RecyclerView recyclerView;
    private RecyclerView.LayoutManager layoutManager;
    private SiteAdapter siteAdapter;

    private ProgressBar progressBar;
    private TextView loadingText;
    private String myName;

    //////////////////////////////Activity Lifecycle///////////////////////////////
    @Override
    protected void onCreate (Bundle saveInstanceState){
        super.onCreate(saveInstanceState);
        setContentView(R.layout.my_sites);

        Bundle bundle = getIntent().getExtras();
        myList = bundle.getString("fieldName");
        myName = bundle.getString("listName");

        ///// ACTION BAR /////

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        if (getSupportActionBar() != null){
            ActionBar actionBar = getSupportActionBar();
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setDisplayShowHomeEnabled(true);
            actionBar.setTitle(myName);
        }

        /////    /////

        progressBar = (ProgressBar) findViewById(R.id.progressBar);
        loadingText = (TextView) findViewById(R.id.tv_loading);

        myDatabase = new MySitesDatabase(this);
        mySitesId = myDatabase.mySiteList(myList);

        mySitesList = new ArrayList<>();

        databaseReference = FirebaseDatabase.getInstance().getReference()
                .child(SITES_CHILD);
        databaseReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.exists()) {
                    Iterable<DataSnapshot> references = dataSnapshot.getChildren();
                    while (references.iterator().hasNext()) {
                        DataSnapshot nextDataReference = references.iterator().next();
                        Site site = nextDataReference.getValue(Site.class);
                        if(mySitesId.contains(site.getId())) {
                            mySitesList.add(site);
                        }
                    }
                }

                if (mySitesList.size() > 0) {
                    progressBar.setVisibility(ProgressBar.INVISIBLE);
                    loadingText.setVisibility(TextView.INVISIBLE);

                    recyclerView = (RecyclerView) findViewById(R.id.image_content);
                    siteAdapter = new SiteAdapter(MySitesActivity.this, mySitesList,
                            R.layout.site_element, true, false, true);
                    layoutManager = new LinearLayoutManager(MySitesActivity.this,
                            LinearLayoutManager.VERTICAL, false);
                    recyclerView.setAdapter(siteAdapter);
                    recyclerView.setLayoutManager(layoutManager);
                    onClickListener();

                } else {
                    MyNotifications.createToastNotification(
                            MySitesActivity.this,
                            getResources().getString(R.string.no_site_in_my_sites) + " " + myName,
                            MyNotifications.LONG_DURATION);
                    finish();
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                MyNotifications.createToastNotification(MySitesActivity.this,
                        R.string.server_conexion_cancelled,
                        MyNotifications.LONG_DURATION);
            }
        });
    }
    /////////////////////////////////////////////////////////////////////////////////

    private void onClickListener(){
        siteAdapter.setItemOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                int position = recyclerView.getChildAdapterPosition(view);
                Site site = mySitesList.get(position);

                Intent intent = new Intent(MySitesActivity.this,
                        SiteDetailActivity.class);
                intent.putExtra("id", site.getId());
                intent.putExtra("name", site.getName());
                intent.putExtra("siteLatitude", site.getsCoordinate().getLatitude());
                intent.putExtra("siteLongitude", site.getsCoordinate().getLongitude());
                intent.putExtra("type", site.getType().toString());
                startActivity(intent);
                finish();
            }
        });
    }

    //////////////////////////////////ActionBar Options//////////////////////////////////

    @Override
    public boolean onOptionsItemSelected(MenuItem item){
        switch(item.getItemId()){
            case android.R.id.home:
                finish();
        }
        return super.onOptionsItemSelected(item);
    }
}
