package com.david.oviedotourist.activities;

import android.content.DialogInterface;
import android.content.Intent;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.support.design.widget.FloatingActionButton;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.Toolbar;
import android.text.Html;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;
import android.widget.Button;
import android.widget.ProgressBar;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.david.oviedotourist.R;
import com.david.oviedotourist.adapters.ImageAdapter;
import com.david.oviedotourist.data.MySitesDatabase;
import com.david.oviedotourist.pojo.GeoCoordinate;
import com.david.oviedotourist.pojo.Site;
import com.david.oviedotourist.utilities.MyNotifications;
import com.david.oviedotourist.utilities.MyScrollView;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.Locale;


/**
 * Created by David on 08/11/2016.
 */

public class SiteDetailActivity extends AppCompatActivity {

    //Images Management//
    private RecyclerView recyclerView;
    private RecyclerView.LayoutManager layoutManager;
    private ImageAdapter imageAdapter;

    //Loading variables//
    private ProgressBar progressBar;
    private TextView loadingText;

    //Data Management//
    private String siteId, siteName;
    private GeoCoordinate coordinates;
    private TextView siteAddress, siteDistance, siteDescription, siteSource;
    private boolean isPanoramic;

    //User lists//
    private MySitesDatabase database;
    private Button myLists;

    //FloatingActionButtons//
    private FloatingActionButton fabToGoogleMaps, fabToNavigation, fabToStreetView;

    private final static String SITE_DETAILS_CHILD = "siteDetails/";
    private final static String FAVOURITE_LIST = "favourite";
    private final static String TO_VISIT_LIST = "visit";
    private final static int SOURCE = R.string.source;
    private final static int METERS = R.string.meters;


    //////////////////////////////Activity Lifecycle///////////////////////////////

    @Override
    protected void onCreate(Bundle saveInstanceState) {
        super.onCreate(saveInstanceState);
        setContentView(R.layout.site_detail);

        Bundle bundle = getIntent().getExtras();

        /////CUSTOM BARS AND COLORS/////

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        RelativeLayout centerBar = (RelativeLayout) findViewById(R.id.center_bar);
        String siteType = bundle.getString("type", "");
        siteName = bundle.getString("name", "");

        toolbar.setBackgroundColor(checkTypeColor(siteType, false));
        centerBar.setBackgroundColor(checkTypeColor(siteType, false));

        setSupportActionBar(toolbar);

        if (getSupportActionBar() != null) {
            ActionBar actionBar = getSupportActionBar();
            actionBar.setDisplayHomeAsUpEnabled(true);
            actionBar.setDisplayShowHomeEnabled(true);
            actionBar.setTitle(siteName);
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            Window window = getWindow();
            window.setStatusBarColor(checkTypeColor(siteType, true));
        }

        /////  /////

        progressBar = (ProgressBar) findViewById(R.id.progressBar);
        loadingText = (TextView) findViewById(R.id.tv_loading);
        siteAddress = (TextView) findViewById(R.id.tv_address);
        siteDistance = (TextView) findViewById(R.id.tv_distance);
        siteDescription = (TextView) findViewById(R.id.tv_description);
        siteSource = (TextView) findViewById(R.id.tv_source);
        fabToGoogleMaps = (FloatingActionButton) findViewById(R.id.fab_google_maps);
        fabToNavigation = (FloatingActionButton) findViewById(R.id.fab_navigation_view);
        fabToStreetView = (FloatingActionButton) findViewById(R.id.fab_street_view);
        fabToStreetView.setVisibility(FloatingActionButton.INVISIBLE);

        siteId = bundle.getString("id");
        coordinates = new GeoCoordinate(bundle.getDouble("siteLatitude", 0),
                bundle.getDouble("siteLongitude", 0));
        final double distance = bundle.getDouble("distance", -1);

        DatabaseReference databaseReference = FirebaseDatabase.getInstance().
                getReference().child(SITE_DETAILS_CHILD + siteId);
        databaseReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.exists()) {
                    Site site = dataSnapshot.getValue(Site.class);

                    siteAddress.setText(site.getAddress());
                    if (distance != -1) {
                        siteDistance.setText(String.format(Locale.getDefault(), "%1.0f",
                                distance).concat(" " + getString(METERS)));
                    }
                    siteDescription.setText(Html.fromHtml(site.getDescription()));
                    siteSource.setText(getString(SOURCE) + " " + site.getSource());

                    isPanoramic = checkIfPanoramic(site.getPanoramic());

                    progressBar.setVisibility(ProgressBar.INVISIBLE);
                    loadingText.setVisibility(TextView.INVISIBLE);

                    recyclerView = (RecyclerView) findViewById(R.id.image_content);
                    imageAdapter = new ImageAdapter(SiteDetailActivity.this, site);
                    recyclerView.setAdapter(imageAdapter);
                    layoutManager = new LinearLayoutManager(SiteDetailActivity.this,
                            LinearLayoutManager.HORIZONTAL, false);
                    recyclerView.setLayoutManager(layoutManager);
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                MyNotifications.createToastNotification(SiteDetailActivity.this,
                        R.string.server_conexion_cancelled, MyNotifications.SHORT_DURATION);
            }
        });

        Animation toGoogleMaps = AnimationUtils.loadAnimation(this, R.anim.zoom_in);
        toGoogleMaps.setStartOffset(200);
        fabToGoogleMaps.startAnimation(toGoogleMaps);
        fabToGoogleMaps.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                fabToGoogleMaps.startAnimation(AnimationUtils.loadAnimation(
                        SiteDetailActivity.this, R.anim.zoom_out));
                Intent intent = new Intent(SiteDetailActivity.this, VirtualTourActivity.class);
                intent.putExtra("id", siteId);
                intent.putExtra("isSite", true);
                startActivity(intent);
            }
        });

        Animation toNavigation = AnimationUtils.loadAnimation(this, R.anim.zoom_in);
        toNavigation.setStartOffset(500);
        fabToNavigation.startAnimation(toNavigation);
        fabToNavigation.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                fabToNavigation.startAnimation(AnimationUtils.loadAnimation(
                        SiteDetailActivity.this, R.anim.zoom_out));
                Intent intent = new Intent(Intent.ACTION_VIEW);
                intent.setData(Uri.parse("google.navigation:q=" + coordinates.getLatitude()
                        + "," + coordinates.getLongitude()));
                startActivity(intent);
            }
        });

        final MyScrollView scroll = (MyScrollView) findViewById(R.id.scroll_view);
        scroll.setOnScrollViewListener(new MyScrollView.OnScrollViewListener() {
            @Override
            public void onScrollChanged(int x, int y, int previousX, int previousY) {
                int lastPositionY, previousPositionY;
                lastPositionY = y <= 0 || previousY <= 0 ? 0 : y;
                previousPositionY = y <= 0 || previousY <= 0 ? 0 : previousY;

                if (lastPositionY > previousPositionY) {
                    fabToGoogleMaps.hide();
                    fabToNavigation.hide();
                    fabToStreetView.hide();
                } else {
                    fabToGoogleMaps.show();
                    fabToNavigation.show();
                    if(isPanoramic) {
                        fabToStreetView.show();
                    }
                }
                Log.d("SCROLL", " Y: " + lastPositionY + " Previous Y: " + previousPositionY);
            }
        });
        /////USER LIST MANAGEMENT/////

        database = new MySitesDatabase(this);

        myLists = (Button) findViewById(R.id.bt_add_site);
        myLists.setActivated(database.searchSite(siteId, FAVOURITE_LIST)
                || database.searchSite(siteId, TO_VISIT_LIST));
        myLists.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                addSiteToList(siteId,
                        getResources().getStringArray(R.array.list_types),
                        new boolean[]{database.searchSite(siteId, FAVOURITE_LIST),
                                database.searchSite(siteId, TO_VISIT_LIST)});
            }
        });

    }

    /////////////////////////////////////////////////////////////////////////////////

    private void addSiteToList(final String siteId, final String[] referenceLists,
                                 boolean[] checkedStates) {

        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        builder.setTitle(getString(R.string.add) + " " + siteName + " " + getString(R.string.to));
        builder.setMultiChoiceItems(referenceLists, checkedStates,
                new DialogInterface.OnMultiChoiceClickListener() {
                    @Override
                    public void onClick(DialogInterface dialogInterface,
                                        int whichButton, boolean isChecked) {
                        if (isChecked) {
                            database.saveSite(SiteDetailActivity.this,
                                    siteId, referenceLists[whichButton]);
                        } else {
                            database.deleteSite(SiteDetailActivity.this,
                                    siteId, referenceLists[whichButton]);
                        }
                    }
                });
        builder.setPositiveButton(android.R.string.ok, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialogInterface, int i) {
                myLists.setActivated(database.searchSite(siteId, FAVOURITE_LIST)
                        || database.searchSite(siteId, TO_VISIT_LIST));
            }
        });

        builder.show();
    }

    private boolean checkIfPanoramic(final String panoramic) {
        if (!panoramic.isEmpty()) {
            Animation toStreetView = AnimationUtils.loadAnimation(this, R.anim.zoom_in);
            toStreetView.setStartOffset(800);
            fabToStreetView.startAnimation(toStreetView);
            fabToStreetView.setVisibility(FloatingActionButton.VISIBLE);
            fabToStreetView.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View view) {
                    fabToStreetView.startAnimation(AnimationUtils.loadAnimation(
                            SiteDetailActivity.this, R.anim.zoom_out));
                    Intent intent = new Intent(SiteDetailActivity.this, StreetViewActivity.class);
                    intent.putExtra("panoId", panoramic);
                    intent.putExtra("name", siteName);
                    startActivity(intent);
                }
            });
            return true;
        }
        return false;
    }

    //////////////////////////////////ActionBar Options//////////////////////////////////

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.secondary_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
                return true;
            case R.id.menu_share:
                sharePlace(siteName);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }

    private void sharePlace(String name) {
        Intent intent = new Intent(Intent.ACTION_SEND);
        intent.setType("text/plain");
        intent.putExtra(Intent.EXTRA_TEXT, getString(R.string.site_text) +
                " " + name);
        startActivity(Intent.createChooser(intent, getString(R.string.send_to)));
    }

    private int checkTypeColor(String string, boolean statusBar) {
        switch (string) {
            case "CULTURE":
                return statusBar ? ContextCompat.getColor(this, R.color.cultureDark)
                        : ContextCompat.getColor(this, R.color.culture);
            case "NATURE":
                return statusBar ? ContextCompat.getColor(this, R.color.natureDark)
                        : ContextCompat.getColor(this, R.color.nature);
            case "ENTERTAINMENT":
                return statusBar ? ContextCompat.getColor(this, R.color.entertainmentDark)
                        : ContextCompat.getColor(this, R.color.entertainment);
            case "GASTRONOMY":
                return statusBar ? ContextCompat.getColor(this, R.color.gastronomyDark)
                        : ContextCompat.getColor(this, R.color.gastronomy);
        }
        return ContextCompat.getColor(this, R.color.colorPrimaryDark);
    }
}
