package com.david.oviedotourist.activities;

import android.annotation.TargetApi;
import android.content.Intent;
import android.content.SharedPreferences;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.VectorDrawable;
import android.os.Build;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.Toolbar;
import android.view.MenuItem;
import android.view.View;
import android.widget.LinearLayout;

import com.david.oviedotourist.R;
import com.david.oviedotourist.pojo.GeoCoordinate;
import com.david.oviedotourist.pojo.Site;
import com.david.oviedotourist.pojo.SiteType;
import com.david.oviedotourist.utilities.MyNotifications;
import com.google.android.gms.maps.CameraUpdateFactory;
import com.google.android.gms.maps.GoogleMap;
import com.google.android.gms.maps.MapFragment;
import com.google.android.gms.maps.OnMapReadyCallback;
import com.google.android.gms.maps.model.BitmapDescriptorFactory;
import com.google.android.gms.maps.model.LatLng;
import com.google.android.gms.maps.model.Marker;
import com.google.android.gms.maps.model.MarkerOptions;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;


/**
 * Created by David on 02/12/2016.
 */

public class VirtualTourActivity extends AppCompatActivity
        implements OnMapReadyCallback, GoogleMap.OnInfoWindowClickListener {

    ///// FIREBASE DATABASE /////
    private static final String SITES_CHILD = "sites/";
    private DatabaseReference databaseReference;
    private boolean isDatabaseLoaded = false;

    ///// DATA MANAGAMENT /////
    private SharedPreferences preferences;
    private List<Site> siteList;
    private Bundle bundle;
    private int sitePosition;

    ///// GOOGLE MAPS /////
    private final static int MAP_ZOOM = 16;
    private GoogleMap map;
    private boolean isGoogleMapLoaded = false;
    private LinearLayout linearLayout;

    //////////////////////////////Activity Lifecycle///////////////////////////////

    @Override
    protected void onCreate(Bundle saveInstanceState) {
        super.onCreate(saveInstanceState);
        setContentView(R.layout.virtual_tour);

        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);

        if (getSupportActionBar() != null) {
            ActionBar actionBar = getSupportActionBar();
            actionBar.setDisplayHomeAsUpEnabled(true);
        }

        linearLayout = (LinearLayout) findViewById(R.id.ly_content);

        preferences = PreferenceManager.getDefaultSharedPreferences(this);

        MapFragment mapFragment = (MapFragment) getFragmentManager()
                .findFragmentById(R.id.virtual_map);
        mapFragment.getMapAsync(this);

        siteList = new ArrayList<>();

        bundle = getIntent().getExtras();

        if (bundle != null && bundle.getBoolean("isEvent", false)) {
            Site eventSite = new Site();
            eventSite.setId(bundle.getString("id", ""));
            eventSite.setName(bundle.getString("name", ""));
            eventSite.setsCoordinate(new GeoCoordinate(bundle.getDouble("eventLatitude", 0),
                    bundle.getDouble("eventLongitude", 0)));
            eventSite.setType(checkEventType(bundle.getString("type", "")));

            siteList.add(eventSite);
            sitePosition = checkSitePosition(eventSite.getId());
            isDatabaseLoaded = !siteList.isEmpty();

            return;
        }

        databaseReference = FirebaseDatabase.getInstance()
                .getReference().child(SITES_CHILD);
        databaseReference.addListenerForSingleValueEvent(new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                if (dataSnapshot.exists()) {
                    Iterable<DataSnapshot> references = dataSnapshot.getChildren();
                    while (references.iterator().hasNext()) {
                        DataSnapshot nextDataReference = references.iterator().next();
                        Site site = nextDataReference.getValue(Site.class);
                        if (checkSiteType(site.getType())) {
                            siteList.add(site);
                        }
                    }
                    isDatabaseLoaded = !siteList.isEmpty();
                    if (isGoogleMapLoaded) {
                        sitePosition = bundle == null ? 0 :
                                checkSitePosition(bundle.getString("id", null));
                        onMapReady(map);
                    }
                }
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                MyNotifications.createToastNotification(VirtualTourActivity.this,
                        R.string.server_conexion_cancelled,
                        MyNotifications.LONG_DURATION);
            }
        });
    }

    //////////////////////OnMapReadyCallback implementation///////////////////////

    @Override
    public void onMapReady(GoogleMap googleMap) {
        map = googleMap;
        isGoogleMapLoaded = true;
        if (isDatabaseLoaded) {
            linearLayout.setVisibility(View.INVISIBLE);
            map.setMapType(GoogleMap.MAP_TYPE_SATELLITE);
            if (bundle != null && bundle.getBoolean("isSite", false)) {
                map.setOnInfoWindowClickListener(this);
            }
            map.moveCamera(CameraUpdateFactory.newLatLngZoom(
                    new LatLng(siteList.get(sitePosition).getsCoordinate()
                            .getLatitude(), siteList.get(sitePosition).getsCoordinate()
                            .getLongitude()), MAP_ZOOM));
            for (Site site : siteList) {
                Bitmap icon;
                icon = checkBitmapIcon(site.getType());
                map.addMarker(new MarkerOptions()
                        .position(new LatLng(site.getsCoordinate().getLatitude(),
                                site.getsCoordinate().getLongitude()))
                        .snippet(site.getSummary())
                        .title(site.getName())
                        .flat(true)
                        .icon(BitmapDescriptorFactory.fromBitmap(icon)));
            }
        }
    }

    /////////////////////OnInfoWindowClickListener implementation//////////////////////

    @Override
    public void onInfoWindowClick(Marker marker) {
        for (Site site : siteList) {
            if (site.getName().equals(marker.getTitle())) {
                Intent intent = new Intent(this, SiteDetailActivity.class);
                intent.putExtra("id", site.getId());
                intent.putExtra("name", site.getName());
                intent.putExtra("siteLatitude", site.getsCoordinate().getLatitude());
                intent.putExtra("siteLongitude", site.getsCoordinate().getLongitude());
                intent.putExtra("type", site.getType().toString());
                startActivity(intent);
                break;
            }
        }
    }

    //////////////////////////////////ActionBar Options//////////////////////////////////

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                finish();
        }
        return super.onOptionsItemSelected(item);
    }

    ///////////////////////////////////////Methods///////////////////////////////////////

    @TargetApi(Build.VERSION_CODES.LOLLIPOP)
    private static Bitmap getBitmap(VectorDrawable vectorDrawable) {
        Bitmap bitmap = Bitmap.createBitmap(vectorDrawable.getIntrinsicWidth(),
                vectorDrawable.getIntrinsicHeight(), Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        vectorDrawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
        vectorDrawable.draw(canvas);
        return bitmap;
    }

    private static Bitmap getBitmap(BitmapDrawable bitmapDrawable) {
        Bitmap bitmap = Bitmap.createBitmap(bitmapDrawable.getIntrinsicWidth(),
                bitmapDrawable.getIntrinsicHeight(), Bitmap.Config.ARGB_8888);
        Canvas canvas = new Canvas(bitmap);
        bitmapDrawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
        bitmapDrawable.draw(canvas);
        return bitmap;
    }

    private int checkSitePosition(String id) {
        for (Site site : siteList) {
            if (site.getId().equals(id)) {
                return siteList.indexOf(site);
            }
        }
        return 0;
    }

    private boolean checkSiteType(SiteType type) {
        Set<String> userSelectedTypes = preferences
                .getStringSet("prf_site_type", null);
        return userSelectedTypes != null && userSelectedTypes.contains(type.toString());
    }

    private SiteType checkEventType(String type) {
        switch (type) {
            case "CULTURE":
                return SiteType.CULTURE;
            case "NATURE":
                return SiteType.NATURE;
            case "ENTERTAINMENT":
                return SiteType.ENTERTAINMENT;
            case "GASTRONOMY":
                return SiteType.GASTRONOMY;
            default:
                return null;
        }
    }

    private Bitmap checkBitmapIcon(SiteType siteType) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            return getBitmap((VectorDrawable) ContextCompat
                    .getDrawable(this, siteType.getIdIcon()));

        }

        return getBitmap((BitmapDrawable) ContextCompat
                .getDrawable(this, siteType.getIdBitmap()));
    }
}