package com.david.oviedotourist.adapters;

import android.content.Context;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.david.oviedotourist.R;
import com.david.oviedotourist.pojo.Site;

import java.util.ArrayList;
import java.util.List;

/**
 * Created by David on 08/11/2016.
 */

public class EventAdapter extends RecyclerView.Adapter<EventAdapter.ViewHolder> {


    private Context context;
    private List<Site> eventList;
    private LayoutInflater layoutInflater;
    private View.OnClickListener onClickListener;


    public EventAdapter(Context context, List<Site> eventList) {
        this.context = context;
        this.eventList = new ArrayList<>(eventList);
        layoutInflater = (LayoutInflater)
                context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }

    /////////////////////////RecyclerView.Adapter<> Implementation//////////////////////////

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = layoutInflater.inflate(R.layout.event_element, parent, false);
        itemView.setOnClickListener(onClickListener);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {

        Glide.with(context)
                .load(eventList.get(position).getImage())
                .diskCacheStrategy(DiskCacheStrategy.SOURCE)
                .fitCenter()
                .centerCrop()
                .into(holder.imageView);

        holder.nameView.setText(eventList.get(position).getName());
        holder.eventDate.setText(eventList.get(position).getEvent().getDuration());
        holder.eventTime.setText(eventList.get(position).getEvent().getTime());
        holder.imageView.setAdjustViewBounds(true);

        switch (eventList.get(position).getType()) {
            case CULTURE:
                holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                        context, R.drawable.ic_culture));

                return;
            case NATURE:
                holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                        context, R.drawable.ic_nature));

                return;
            case ENTERTAINMENT:
                holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                        context, R.drawable.ic_entertainment));

                return;
            case GASTRONOMY:
                holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                        context, R.drawable.ic_gastronomy));
        }
    }


    @Override
    public int getItemCount() {
        return eventList.size();
    }

    /////////////////////////////////////////////////////////////////////////////////////////

    public void setItemOnClickListener(View.OnClickListener onClickListener) {
        this.onClickListener = onClickListener;
    }

    public void swap(List<Site> newList) {
        eventList.clear();
        eventList.addAll(newList);
        notifyDataSetChanged();
    }

/////////////////////////////////////////////////////////////////////////////////////////

    static class ViewHolder extends RecyclerView.ViewHolder {

        ImageView imageView, iconView;
        TextView nameView, eventDate, eventTime;

        ViewHolder(View itemView) {
            super(itemView);
            imageView = (ImageView) itemView.findViewById(R.id.event_image);
            iconView = (ImageView) itemView.findViewById(R.id.event_icon);
            nameView = (TextView) itemView.findViewById(R.id.event_name);
            eventDate = (TextView) itemView.findViewById(R.id.event_date);
            eventTime = (TextView) itemView.findViewById(R.id.event_time);
        }
    }
}