package com.david.oviedotourist.adapters;

import android.content.Context;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.david.oviedotourist.R;
import com.david.oviedotourist.pojo.Site;

import java.util.Random;

/**
 * Created by David on 08/11/2016.
 */

public class ImageAdapter extends RecyclerView.Adapter<ImageAdapter.ViewHolder> {


    private Context context;
    private Site site;
    private LayoutInflater layoutInflater;

    public ImageAdapter(Context context, Site site) {
        this.context = context;
        this.site = site;
        layoutInflater = (LayoutInflater)
                context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }

    /////////////////////////RecyclerView.Adapter<> Implementation//////////////////////////

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = layoutInflater.inflate(R.layout.image_element, parent, false);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        String image = site.getPhotos().get(position).getPhoto();
        String name = site.getPhotos().get(position).getPhotoName();

        int imageSize = context.getResources().getDimensionPixelSize(R.dimen.image_size);
        int randomSize = new Random().nextInt(128) + 96;
        int imageWidth = imageSize + randomSize;

        Glide.with(context)
                .load(image)
                .diskCacheStrategy(DiskCacheStrategy.SOURCE)
                .fitCenter()
                .override(imageWidth, imageSize)
                .centerCrop()
                .into(holder.imageView);

        holder.nameView.setText(name);
        holder.imageView.setAdjustViewBounds(true);
    }

    @Override
    public int getItemCount() {
        return site.getPhotos().size();
    }

    /////////////////////////////////////////////////////////////////////////////////////////

    static class ViewHolder extends RecyclerView.ViewHolder {

        ImageView imageView;
        TextView nameView;

        ViewHolder(View itemView) {
            super(itemView);
            imageView = (ImageView) itemView.findViewById(R.id.site_image);
            nameView = (TextView) itemView.findViewById(R.id.site_name);
        }
    }
}