package com.david.oviedotourist.adapters;

import android.content.Context;
import android.support.v4.content.ContextCompat;
import android.support.v7.widget.CardView;
import android.support.v7.widget.RecyclerView;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.bumptech.glide.Glide;
import com.bumptech.glide.load.engine.DiskCacheStrategy;
import com.david.oviedotourist.R;
import com.david.oviedotourist.pojo.Site;
import com.david.oviedotourist.pojo.SiteType;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 * Created by David on 08/11/2016.
 */

public class SiteAdapter extends RecyclerView.Adapter<SiteAdapter.ViewHolder> {


    private Context context;
    private List<Site> siteList;
    private int layout;
    private LayoutInflater layoutInflater;
    private View.OnClickListener onClickListener;
    private boolean hasSummary;
    private boolean colorful;
    private boolean uniformSize;

    public SiteAdapter(Context context, List<Site> siteList, int layout, boolean hasSummary,
                       boolean colorful, boolean uniformSize){
        this.context = context;
        this.siteList = new ArrayList<>(siteList);
        this.layout = layout;
        this.hasSummary = hasSummary;
        this.colorful = colorful;
        this.uniformSize = uniformSize;
        layoutInflater = (LayoutInflater)
                context.getSystemService(Context.LAYOUT_INFLATER_SERVICE);
    }

    /////////////////////////RecyclerView.Adapter<> Implementation//////////////////////////

    @Override
    public ViewHolder onCreateViewHolder(ViewGroup parent, int viewType) {
        View itemView = layoutInflater.inflate(layout, parent, false);
        itemView.setOnClickListener(onClickListener);
        return new ViewHolder(itemView);
    }

    @Override
    public void onBindViewHolder(ViewHolder holder, int position) {
        String image = siteList.get(position).getImage();
        String name = siteList.get(position).getName();
        SiteType type = siteList.get(position).getType();

        int imageSize, imageWidth, imageHeight;

        if(!uniformSize) {
            imageSize = context.getResources().getDimensionPixelSize(R.dimen.image_size);
            int randomSize = new Random().nextInt(128) + 96;
            imageWidth = imageSize;
            imageHeight = imageSize + randomSize;

        } else {
            imageSize = context.getResources().getDimensionPixelSize(R.dimen.site_image_size);
            imageWidth = imageSize;
            imageHeight = imageSize;
        }

        Glide.with(context)
                .load(image)
                .diskCacheStrategy(DiskCacheStrategy.SOURCE)
                .fitCenter()
                .override(imageWidth, imageHeight)
                .centerCrop()
                .into(holder.imageView);

        holder.nameView.setText(name);
        holder.imageView.setAdjustViewBounds(true);

        if (hasSummary) {
            holder.summaryView.setText(siteList.get(position).getSummary());
        }

        if (type != null){
            switch (type){
                case CULTURE:
                    holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                            context, R.drawable.ic_culture));
                    if (colorful) {
                        holder.cardView.setBackgroundColor(ContextCompat.getColor(
                                context, type.getIdColor()));
                    }
                    return;
                case NATURE:
                    holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                            context, R.drawable.ic_nature));
                    if (colorful) {
                        holder.cardView.setBackgroundColor(ContextCompat.getColor(
                                context, type.getIdColor()));
                    }
                    return;
                case ENTERTAINMENT:
                    holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                            context, R.drawable.ic_entertainment));
                    if (colorful) {
                        holder.cardView.setBackgroundColor(ContextCompat.getColor(
                                context, type.getIdColor()));
                    }
                    return;
                case GASTRONOMY:
                    holder.iconView.setImageDrawable(ContextCompat.getDrawable(
                            context, R.drawable.ic_gastronomy));
                    if (colorful) {
                        holder.cardView.setBackgroundColor(ContextCompat.getColor(
                                context, type.getIdColor()));
                    }
            }
        }
    }

    @Override
    public int getItemCount() {
        return siteList.size();
    }

    /////////////////////////////////////////////////////////////////////////////////////////

    public void setItemOnClickListener (View.OnClickListener onClickListener){
        this.onClickListener = onClickListener;
    }

    public void swap (List<Site> newList){
        siteList.clear();
        siteList.addAll(newList);
        notifyDataSetChanged();
    }

    /////////////////////////////////////////////////////////////////////////////////////////

    static class ViewHolder extends RecyclerView.ViewHolder {

        ImageView imageView, iconView;
        TextView nameView, summaryView;
        CardView cardView;

        ViewHolder(View itemView){
            super(itemView);
            imageView = (ImageView) itemView.findViewById(R.id.site_image);
            iconView = (ImageView) itemView.findViewById(R.id.site_icon);
            nameView = (TextView) itemView.findViewById(R.id.site_name);
            cardView = (CardView) itemView.findViewById(R.id.card_view);
            summaryView =(TextView) itemView.findViewById(R.id.site_summary);
        }
    }
}