package com.david.oviedotourist.services;

import android.Manifest;
import android.app.Service;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.os.IBinder;
import android.support.annotation.Nullable;
import android.support.v4.content.ContextCompat;
import android.support.v4.content.LocalBroadcastManager;


/**
 * Created by David on 13/11/2016.
 */

public class LocationService extends Service
        implements LocationListener {

    private static final long INTERVAL_BETWEEN_LOCATION_UPDATES = 30 * 1000; //milliseconds
    private static final float DISTANCE_BETWEEN_LOCATION_UPDATES = 50; //meters
    private static final long TIME_BETWEEN_LOCATION_UPDATES = 2 * 60 * 1000; //milliseconds
    public static final String NEW_LOCATION_AVAILABLE = "NewLocationAvailableFromLocationService";
    public static final String PROVIDER_AVAILABLE = "ProviderAvailable";

    private LocationManager locationManager;
    private static Location newLocation;
    private static boolean isProviderEnabled;
    private LocalBroadcastManager broadcastManager;


    /////////////////////////////////////Service LifeCycle///////////////////////////////////////

    @Override
    public void onCreate() {
        super.onCreate();
        broadcastManager = LocalBroadcastManager.getInstance(this);
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int idStart) {
        providerActivation(this);
        return START_STICKY;
    }

    @Override
    public void onDestroy() {
        providerDeactivation();
    }

    @Nullable
    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }


    ////////////////////////////////////////////////////////////////////////////////////////////

    /////////////////////////////////////Location Methods///////////////////////////////////////

    private void providerActivation(LocationListener locationListener) {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                == PackageManager.PERMISSION_GRANTED) {
            locationManager = (LocationManager) getSystemService(LOCATION_SERVICE);
            if (locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)) {
                locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER,
                        INTERVAL_BETWEEN_LOCATION_UPDATES,
                        DISTANCE_BETWEEN_LOCATION_UPDATES,
                        locationListener);
                newLocation = updatePosition(locationManager
                        .getLastKnownLocation(LocationManager.GPS_PROVIDER));
            }
            if (locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)) {
                locationManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER,
                        INTERVAL_BETWEEN_LOCATION_UPDATES,
                        DISTANCE_BETWEEN_LOCATION_UPDATES,
                        locationListener);
                newLocation = updatePosition(locationManager
                        .getLastKnownLocation(LocationManager.NETWORK_PROVIDER));
            }

            isProviderEnabled = (locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
                    || locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER));

            sendProviderResult(isProviderEnabled);

            if (newLocation != null) {
                sendLocationResult(newLocation.getLatitude(), newLocation.getLongitude());
            }

        }
    }

    private void providerDeactivation() {
        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_FINE_LOCATION)
                == PackageManager.PERMISSION_GRANTED) {
            locationManager.removeUpdates(this);
        }
    }

    private Location updatePosition(Location location) {
        if (location != null && (newLocation == null
                || location.getAccuracy() < 2 * newLocation.getAccuracy()
                || location.getTime() - newLocation.getTime() > TIME_BETWEEN_LOCATION_UPDATES)) {
            return location;
        }
        return newLocation;
    }

    //////////////////////////////LocationListener implementation///////////////////////////////

    @Override
    public void onLocationChanged(Location location) {
        newLocation = updatePosition(location);
        sendLocationResult(newLocation.getLatitude(), newLocation.getLongitude());
    }

    @Override
    public void onStatusChanged(String s, int i, Bundle bundle) {
        providerActivation(this);
    }

    @Override
    public void onProviderEnabled(String s) {
        isProviderEnabled = true;
        providerActivation(this);
    }

    @Override
    public void onProviderDisabled(String s) {
        isProviderEnabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
                || locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
        sendProviderResult(isProviderEnabled);
    }

    ////////////////////////////////////////////////////////////////////////////////////////////

    //////////////////////////////////Broadcast methods////////////////////////////////////////

    public void sendLocationResult(double latitude, double longitude) {
        Intent intent = new Intent(NEW_LOCATION_AVAILABLE);
        intent.putExtra("myLatitude", latitude);
        intent.putExtra("myLongitude", longitude);
        broadcastManager.sendBroadcast(intent);
    }

    public void sendProviderResult(boolean provider) {
        Intent intent = new Intent(PROVIDER_AVAILABLE);
        intent.putExtra("provider", provider);
        broadcastManager.sendBroadcast(intent);
    }

    ////////////////////////////////////////////////////////////////////////////////////////////
}
